using System;
using System.Collections.Generic;
using System.Text;

namespace BarCodes
{
	public static class CBEncoder
	{
        public static string CodifyCode39(string cb)
        {
            return "*" + cb + "*";
        }
		private static string CtrlEAN13(string cb)
		{
			int i = 11;
			int checksum = 0;
			while (i >= 0) 
			{
				checksum += int.Parse(cb[i].ToString());
				i -= 2;
			}
			checksum *= 3;
			i = 10;
			while (i >= 0) 
			{
				checksum += int.Parse(cb[i].ToString());
				i -= 2;
			}
			return cb + (char)(48 + (10 - checksum % 10) % 10);
		}
		public static string CodifyEAN13(string cb) 
		{
			string result = "";
            cb = cb.PadLeft(12, '0');
            if (cb.Length == 12)
            {
                long val = long.Parse(cb);  // Test if cb is a number
                if (cb.Length == 12)
                {
                    cb = CtrlEAN13(cb);    // calculate control digit 
                }
                string barcode = cb[0].ToString() + (char)(65 + int.Parse(cb[1].ToString()));
                int first = int.Parse(cb[0].ToString());
                int offset;
                for (int i = 2; i < 7; i++)
                {
                    offset = 75;
                    switch (i)
                    {
                        case 2:
                            switch (first)
                            {
                                case 0:
                                case 1:
                                case 2:
                                case 3:
                                    offset = 65;
                                    break;
                            }
                            break;
                        case 3:
                            switch (first)
                            {
                                case 0:
                                case 4:
                                case 7:
                                case 8:
                                    offset = 65;
                                    break;
                            }
                            break;
                        case 4:
                            switch (first)
                            {
                                case 0:
                                case 1:
                                case 4:
                                case 5:
                                case 9:
                                    offset = 65;
                                    break;
                            }
                            break;
                        case 5:
                            switch (first)
                            {
                                case 0:
                                case 2:
                                case 5:
                                case 6:
                                case 7:
                                    offset = 65;
                                    break;
                            }
                            break;
                        case 6:
                            switch (first)
                            {
                                case 0:
                                case 3:
                                case 6:
                                case 8:
                                case 9:
                                    offset = 65;
                                    break;
                            }
                            break;
                    }
                    barcode += (char)(offset + int.Parse(cb[i].ToString()));
                }
                barcode += "*";
                for (int i = 7; i < 13; i++)
                {
                    barcode += (char)(97 + int.Parse(cb[i].ToString()));
                }
                barcode += "+";
                result = barcode;
            }
            else
            {
                throw new Exception("Bad length");
            }
			return result;
		}
		private static void TestNum(string cb, int i, ref int mini)
		{
			mini--;
			if ((i + mini) < cb.Length) 
			{
				while (mini >= 0) 
				{
					if (((byte)(cb[i + mini]) < 48) || ((byte)(cb[i + mini]) > 57)) 
					{
						break;
					}
					mini--;
				}
			}
		}
		public static string CodifyCode128(string cb)
		{
			string result = "";
			bool ok = true;
			if (cb.Length > 0)
			{
				for (int i = 0; i < cb.Length; i++)
				{
					if ((((byte)(cb[i]) >= 32) && ((byte)(cb[i]) <= 126)) || ((byte)(cb[i]) == 203))
					{
						ok = true;
					}
					else
					{
						ok = false;
						break;
					}
				}
				bool tableb = true;
                if (ok)
                {
                    int i = 0;
                    int mini = 0;
                    int dummy = 0;
                    while (i < cb.Length)
                    {
                        if (tableb)
                        {
                            if (((i == 0) || ((i + 3) == (cb.Length - 1))))
                            {
                                mini = 4;
                            }
                            else
                            {
                                mini = 6;
                            }
                            TestNum(cb, i, ref mini);
                            if (mini < 0)
                            {
                                if (i == 0)
                                {
                                    result = ((char)(210)).ToString();
                                }
                                else
                                {
                                    result += (char)(204);
                                }
                                tableb = false;
                            }
                            else
                            {
                                if (i == 0)
                                {
                                    result = ((char)(209)).ToString();
                                }
                            }
                        }
                        if (!tableb)
                        {
                            mini = 2;
                            TestNum(cb, i, ref mini);
                            if (mini < 0)
                            {
                                dummy = int.Parse(cb.Substring(i, 2));
                                if (dummy < 95)
                                {
                                    dummy += 32;
                                }
                                else
                                {
                                    dummy += 105;
                                }
                                result += (char)(dummy);
                                i += 2;
                            }
                            else
                            {
                                result += (char)(205);
                                tableb = true;
                            }
                        }
                        if (tableb)
                        {
                            result += cb[i].ToString();
                            i++;
                        }
                    }
                    int checksum = 0;
                    for (i = 0; i < result.Length; i++)
                    {
                        dummy = (int)(result[i]);
                        if (dummy < 127)
                        {
                            dummy -= 32;
                        }
                        else
                        {
                            dummy -= 105;
                        }
                        if (i == 0)
                        {
                            checksum = dummy;
                        }
                        checksum = (checksum + i * dummy) % 103;
                    }
                    if (checksum < 95)
                    {
                        checksum += 32;
                    }
                    else
                    {
                        checksum += 105;
                    }
                    result += (((char)(checksum)).ToString() + ((char)(211)).ToString());
                }
                else
                {
                    throw new Exception("Character not valid")
                }
			}
			return result;
		}
	}
}
